﻿Imports System.Net.Http
Imports <xmlns:dc="http://purl.org/dc/elements/1.1/">

'Represent a single content in the RSS feed
Public Class FeedItem

    'Properties representing information
    'which is common to any RSS feed
    Public Property Title As String = String.Empty
    Public Property Author As String = String.Empty
    Public Property Description As String = String.Empty
    Public Property PubDate As DateTimeOffset
    Public Property Link As Uri

    'Return a collection of FeedItem objects from a RSS feed
    Public Shared Async Function ParseFeedAsync(feedUrl As String) As _
      Task(Of IEnumerable(Of FeedItem))
        Dim client As New HttpClient

        'Download the feed content as a string
        Dim result = Await client.GetStringAsync(New Uri(feedUrl, UriKind.Absolute))

        'If no result, throw an exception
        If result Is Nothing Then
            Throw New InvalidOperationException("The specified URL returned a null result")
        Else
            'LINQ to XML: Convert the returned string
            'into an XDocument object
            Dim document = XDocument.Parse(result)

            'Execute a LINQ query over the XML document
            'and return a collection of FeedItem objects
            Dim query = From item In document...<item>
                        Select New FeedItem With {
                        .Title = item.<title>.Value,
                        .Author = item.<dc:creator>.Value,
                        .Description = item.<description>.Value,
                        .PubDate = DateTimeOffset.Parse(item.<pubDate>.Value),
                        .Link = New Uri(item.<link>.Value)}

            Return query
        End If
    End Function
End Class
