<DiagnosticAnalyzer(LanguageNames.VisualBasic)>
Public Class FeedLibraryAnalyzerAnalyzer
    Inherits DiagnosticAnalyzer


    ' You can change these strings in the Resources.resx file. If you do not want your analyzer to be localize-able, you can use regular strings for Title and MessageFormat.
    Public Const DiagnosticId = "RSS001"
    Friend Shared ReadOnly Title As String = "RSS URL analysis"
    Friend Shared ReadOnly MessageFormat As String = "URL is invalid"
    Friend Shared ReadOnly Description As String = "Provides live analysis for the FeedLibrary APIs"
    Friend Const Category = "Syntax"


    Private Shared Rule As New DiagnosticDescriptor(DiagnosticId, Title, MessageFormat, Category, DiagnosticSeverity.Warning, isEnabledByDefault:=True, description:=Description)

    Public Overrides ReadOnly Property SupportedDiagnostics As ImmutableArray(Of DiagnosticDescriptor)
        Get
            Return ImmutableArray.Create(Rule)
        End Get
    End Property

    Public Overrides Sub Initialize(context As AnalysisContext)

        ' Register an action when compilation starts
        context.
            RegisterCompilationStartAction(Sub(ctx As CompilationStartAnalysisContext)
                                               'Detect if the type metadata 
                                               'exists in the compilation context
                                               Dim myLibraryType =
                                                       ctx.Compilation.
                                                       GetTypeByMetadataName("FeedLibrary.FeedItem")
                                               'If not, return 
                                               '(no reference to the library)
                                               If myLibraryType Is Nothing _
                                                      Then Return
                                               'Register an action against 
                                               'an InvocationExpression
                                               ctx.RegisterSyntaxNodeAction(
                                                   AddressOf AnalyzeMethodInvocation,
                                                   SyntaxKind.InvocationExpression)
                                           End Sub)
    End Sub

    Private Shared Sub AnalyzeMethodInvocation(context As SyntaxNodeAnalysisContext)
        'Check if the type metadata exists in the current context
        'If context.SemanticModel.Compilation.
        '    GetTypeByMetadataName("FeedLibrary.FeedItem") Is Nothing Then Return

        'Convert the current syntax node into an InvocationExpressionSyntax
        'which represents a method call
        Dim invocationExpr = CType(context.Node, InvocationExpressionSyntax)

        'Convert the associated expression into a MemberAccessExpressionSyntax
        'which represents a method's information
        'If the expression Is Not a MemberAccessExpressionSyntax, return
        If TypeOf invocationExpr.Expression IsNot MemberAccessExpressionSyntax Then Return
        Dim memberAccessExpr = DirectCast(invocationExpr.Expression,
            MemberAccessExpressionSyntax)

        'If the method name Is Not ParseFeedAsync, return
        If memberAccessExpr?.Name.ToString <> "ParseFeedAsync" Then Return

        'If the method name is ParseFeedAsync, check for the symbol info
        'and see if the return type matches
        Dim memberSymbol = TryCast(context.SemanticModel.
            GetSymbolInfo(memberAccessExpr).Symbol, IMethodSymbol)
        If memberSymbol Is Nothing Then Return
        Dim result = memberSymbol.ToString

        If Not memberSymbol?.ReturnType.ToString =
            "System.Threading.Tasks.Task(Of System.Collections.Generic.IEnumerable(Of FeedLibrary.FeedItem))" _
            Then Return

        'Check if the method call has the required argument number
        Dim argumentList = invocationExpr.ArgumentList
        If argumentList?.Arguments.Count <> 1 Then Return

        'Convert the expression for the first method argument into
        'a LiteralExpressionSyntax. If null, return
        Dim urlLiteral = DirectCast(invocationExpr?.ArgumentList?.Arguments(0)?.GetExpression,
            LiteralExpressionSyntax)
        If urlLiteral Is Nothing Then Return

        'Convert the actual value for the method argument into string
        'If null, return
        Dim urlLiteralOpt = context.SemanticModel.GetConstantValue(urlLiteral)

        Dim urlValue = DirectCast(urlLiteralOpt.Value, String)
        If urlValue Is Nothing Then Return

        'If the URL Is Not well-formed, create a diagnostic
        If Uri.IsWellFormedUriString(urlValue, UriKind.Absolute) = False Then
            Dim diagn = Diagnostic.Create(Rule, urlLiteral.GetLocation,
                        "The specified parameter Is Not a valid RSS feed")
            context.ReportDiagnostic(diagn)
        End If
    End Sub

End Class